@@ -1,5 +1,5 @@
 # Relational
-# Copyright (C) 2008  Salvo "LtWorf" Tomaselli
+# Copyright (C) 2008-2016  Salvo "LtWorf" Tomaselli
 #
 # Relation is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@ import http.client
 import urllib.parse
 import os.path
 import pickle
+import base64
 
 from relational.relation import relation
 from relational import parser
@@ -41,9 +42,11 @@ def send_survey(data):
     headers = {"Content-type":
                "application/x-www-form-urlencoded", "Accept": "text/plain"}
     connection = http.client.HTTPConnection('feedback-ltworf.appspot.com')
-    connection.request("POST", "/feedback/relational", params, headers)
-
-    return connection.getresponse()
+    try:
+        connection.request("POST", "/feedback/relational", params, headers)
+        return connection.getresponse().status
+    except:
+        return 0
 
 
 def check_latest_version():
@@ -51,8 +54,11 @@ def check_latest_version():
     Heavely dependent on server and server configurations
     not granted to work forever.'''
     connection = http.client.HTTPConnection('feedback-ltworf.appspot.com')
-    connection.request("GET", "/version/relational")
-    r = connection.getresponse()
+    try:
+        connection.request("GET", "/version/relational")
+        r = connection.getresponse()
+    except:
+        return None
 
     # html
     s = r.read()
@@ -98,7 +104,7 @@ class UserInterface (object):
             with open(filename, 'w') as f:
                 pickle.dump(self.relations, f)
                 return None
-        return pickle.dumps(self.relations)
+        return base64.b64encode(pickle.dumps(self.relations)).decode()
 
     def session_restore(self, session=None, filename=None):
         '''
@@ -108,7 +114,7 @@ class UserInterface (object):
         '''
         if session:
             try:
-                self.relations = pickle.loads(session)
+                self.relations = pickle.loads(base64.b64decode(session))
             except:
                 pass
         elif filename:
