@@ -1,5 +1,5 @@
 # Relational
-# Copyright (C) 2009  Salvo "LtWorf" Tomaselli
+# Copyright (C) 2009-2016  Salvo "LtWorf" Tomaselli
 #
 # Relational is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -53,6 +53,37 @@ RENAME = parser.RENAME
 ARROW = parser.ARROW
 
 
+def find_duplicates(node, dups=None):
+    '''
+    Finds repeated subtrees in a parse
+    tree.
+    '''
+    if dups is None:
+        dups = {}
+    dups[str(node)] = node
+
+
+
+def replace_leaves(node, context):
+    '''
+    Node is a parsed tree
+    context is a dictionary containing
+    parsed trees as values.
+
+    If a name appearing in node appears
+    also in context, the parse tree is
+    modified to replace the node with the
+    subtree found in context.
+    '''
+    if node.kind == parser.UNARY:
+        replace_leaves(node.child, context)
+    elif node.kind == parser.BINARY:
+        replace_leaves(node.left, context)
+        replace_leaves(node.right, context)
+    elif node.name in context:
+        replace_node(node, context[node.name])
+
+
 def replace_node(replace, replacement):
     '''This function replaces "replace" node with the node "with",
     the father of the node will now point to the with node'''
@@ -133,7 +164,7 @@ def futile_union_intersection_subtraction(n):
     changes = 0
 
     # Union and intersection of the same thing
-    if n.name in (UNION, INTERSECTION) and n.left == n.right:
+    if n.name in (UNION, INTERSECTION, JOIN, JOIN_LEFT, JOIN_RIGHT, JOIN_FULL) and n.left == n.right:
         changes = 1
         replace_node(n, n.left)
 
@@ -546,14 +577,14 @@ def projection_and_union(n, rels):
     if A and B are union compatible
     '''
     changes = 0
-    if n.name in {UNION, INTERSECTION, DIFFERENCE} and \
+    if n.name == UNION and \
             n.left.name == PROJECTION and \
             n.right.name == PROJECTION and \
             set(n.left.child.result_format(rels)) == set(n.right.child.result_format(rels)):
         newchild = parser.Node()
 
         newchild.kind = parser.BINARY
-        newchild.name = n.name
+        newchild.name = UNION
         newchild.left = n.left.child
         newchild.right = n.right.child
 
@@ -573,7 +604,7 @@ def selection_and_product(n, rels):
     i contains attributes belonging to Q and l contains attributes belonging to both'''
     changes = 0
 
-    if n.name == SELECTION and n.child.name in (PRODUCT, JOIN, JOIN_LEFT, JOIN_RIGHT, JOIN_FULL):
+    if n.name == SELECTION and n.child.name in (PRODUCT, JOIN):
         l_attr = n.child.left.result_format(rels)
         r_attr = n.child.right.result_format(rels)
 
