@@ -1,5 +1,5 @@
 # Relational
-# Copyright (C) 2008  Salvo "LtWorf" Tomaselli
+# Copyright (C) 2008-2016  Salvo "LtWorf" Tomaselli
 #
 # Relational is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +25,8 @@
 
 from relational import optimizations
 from relational import parser
+from relational import querysplit
+from relational.maintenance import UserInterface
 
 
 # Stuff that was here before, keeping it for compatibility
@@ -39,7 +41,28 @@ tree = parser.tree
 # End of the stuff
 
 
-def optimize_all(expression, rels, specific=True, general=True, debug=None):
+def optimize_program(code, rels):
+    '''
+    Optimize an entire program, composed by multiple expressions
+    and assignments.
+    '''
+    lines = code.split('\n')
+    context = {}
+
+    for line in  lines:
+        line = line.strip()
+        if line.startswith(';') or not line:
+            continue
+        res, query = UserInterface.split_query(line)
+        last_res = res
+        parsed = parser.tree(query)
+        optimizations.replace_leaves(parsed, context)
+        context[res] = parsed
+    node = optimize_all(context[last_res], rels, tostr=False)
+    return querysplit.split(node, rels)
+
+
+def optimize_all(expression, rels, specific=True, general=True, debug=None,tostr=True):
     '''This function performs all the available optimizations.
 
     expression : see documentation of this module
@@ -78,7 +101,10 @@ def optimize_all(expression, rels, specific=True, general=True, debug=None):
                 if res != 0 and dbg:
                     debug.append(str(n))
                 total += res
-    return n.__str__()
+    if tostr:
+        return str(n)
+    else:
+        return n
 
 
 def specific_optimize(expression, rels):
