@@ -1,5 +1,5 @@
 # Relational
-# Copyright (C) 2008  Salvo "LtWorf" Tomaselli
+# Copyright (C) 2008-2016  Salvo "LtWorf" Tomaselli
 #
 # Relational is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -194,7 +194,7 @@ class Node (object):
                 prop = '{\"%s\"}' % prop.replace(
                     ',', '\",\"').replace(ARROW, '\":\"').replace(' ', '')
             else:  # Selection
-                prop = '\"%s\"' % prop
+                prop = repr(prop)
 
             return '%s.%s(%s)' % (self.child.toPython(), op_functions[self.name], prop)
         return self.name
@@ -277,11 +277,8 @@ class Node (object):
         elif (self.kind == UNARY):
             return self.name + " " + self.prop + " (" + self.child.__str__() + ")"
         elif (self.kind == BINARY):
-            if self.left.kind == RELATION:
-                le = self.left.__str__()
-            else:
-                le = "(" + self.left.__str__() + ")"
-            if self.right.kind == RELATION:
+            le = self.left.__str__()
+            if self.right.kind != BINARY:
                 re = self.right.__str__()
             else:
                 re = "(" + self.right.__str__() + ")"
@@ -294,7 +291,21 @@ def _find_matching_parenthesis(expression, start=0, openpar=u'(', closepar=u')')
     close parenthesis to the 1st open parenthesis found
     starting from start (0 by default)'''
     par_count = 0  # Count of parenthesis
+
+    string = False
+    escape = False
+
     for i in range(start, len(expression)):
+        if expression[i] == '\'' and not escape:
+            string = not string
+        if expression[i] == '\\' and not escape:
+            escape = True
+        else:
+            escape = False
+        if string:
+            continue
+
+
         if expression[i] == openpar:
             par_count += 1
         elif expression[i] == closepar:
@@ -302,6 +313,30 @@ def _find_matching_parenthesis(expression, start=0, openpar=u'(', closepar=u')')
             if par_count == 0:
                 return i  # Closing parenthesis of the parameter
 
+def _find_token(haystack, needle):
+    '''
+    Like the string function find, but
+    ignores tokens that are within a string
+    literal.
+    '''
+    r = -1
+    string = False
+    escape = False
+
+    for i in range(len(haystack)):
+        if haystack[i] == '\'' and not escape:
+            string = not string
+        if haystack[i] == '\\' and not escape:
+            escape = True
+        else:
+            escape = False
+        if string:
+            continue
+
+        if haystack[i:].startswith(needle):
+            return i
+    return r
+
 
 def tokenize(expression):
     '''This function converts a relational expression into a list where
@@ -334,7 +369,7 @@ def tokenize(expression):
                 par = expression.find(
                     '(', _find_matching_parenthesis(expression))
             else:  # Expression without parenthesis, so adding what's between start and parenthesis as whole
-                par = expression.find('(')
+                par = _find_token(expression, '(')
 
             items.append(expression[:par].strip())
                          # Inserting parameter of the operator
