@@ -1,5 +1,5 @@
 # Relational
-# Copyright (C) 2008  Salvo "LtWorf" Tomaselli
+# Copyright (C) 2008-2016  Salvo "LtWorf" Tomaselli
 #
 # Relational is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -117,7 +117,7 @@ class Relation (object):
             raise TypeError('Expected an instance of the same class')
         elif self.header == other.header:
             return other
-        elif self.header.sharedAttributes(other.header) == len(self.header):
+        elif len(self.header) == len(other.header) and self.header.sharedAttributes(other.header) == len(self.header):
             return other.projection(self.header)
         raise TypeError('Relations differ: [%s] [%s]' % (
             ','.join(self.header), ','.join(other.header)
@@ -130,7 +130,10 @@ class Relation (object):
         newt = relation()
         newt.header = Header(self.header)
 
-        c_expr = compile(expr, 'selection', 'eval')
+        try:
+            c_expr = compile(expr, 'selection', 'eval')
+        except:
+            raise Exception('Failed to compile expression: %s' % expr)
 
         for i in self.content:
             # Fills the attributes dictionary with the values of the tuple
@@ -531,7 +534,10 @@ class Header(tuple):
 
     def getAttributesId(self, param):
         '''Returns a list with numeric index corresponding to field's name'''
-        return [self.index(i) for i in param]
+        try:
+            return [self.index(i) for i in param]
+        except ValueError as e:
+            raise Exception('One of the fields is not in the relation: %s' % ','.join(param))
 
 # Backwards compatibility
 relation = Relation
