@@ -1,5 +1,5 @@
 # Relational
-# Copyright (C) 2008  Salvo "LtWorf" Tomaselli
+# Copyright (C) 2008-2016  Salvo "LtWorf" Tomaselli
 #
 # Relational is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@ class relForm(QtWidgets.QMainWindow):
         self.About = None
         self.Survey = None
         self.undo = None  # UndoQueue for queries
+        self.undo_program = None
         self.selectedRelation = None
         self.ui = maingui.Ui_MainWindow()
         self.user_interface = UserInterface()
@@ -110,7 +111,9 @@ class relForm(QtWidgets.QMainWindow):
         from relational import maintenance
         online = maintenance.check_latest_version()
 
-        if online > version:
+        if online is None:
+            r = QtWidgets.QApplication.translate("Form", "Network error")
+        elif online > version:
             r = QtWidgets.QApplication.translate(
                 "Form", "New version available online: %s." % online)
         elif online == version:
@@ -147,6 +150,19 @@ class relForm(QtWidgets.QMainWindow):
         if self.undo != None:
             self.ui.txtQuery.setText(self.undo)
 
+    def undoOptimizeProgram(self):
+        if self.undo_program:
+            self.ui.txtMultiQuery.setPlainText(self.undo_program)
+
+    def optimizeProgram(self):
+        self.undo_program = self.ui.txtMultiQuery.toPlainText()
+        result = optimizer.optimize_program(
+            self.ui.txtMultiQuery.toPlainText(),
+            self.user_interface.relations
+        )
+        self.ui.txtMultiQuery.setPlainText(result)
+
+
     def optimize(self):
         '''Performs all the possible optimizations on the query'''
         self.undo = self.ui.txtQuery.text()  # Storing the query in undo list
@@ -190,6 +206,9 @@ class relForm(QtWidgets.QMainWindow):
             else:
                 query = self.ui.txtQuery.text()
 
+            if not query.strip():
+                return
+
             try:
                 self.selectedRelation = self.user_interface.multi_execute(query)
             except Exception as e:
@@ -253,7 +272,7 @@ class relForm(QtWidgets.QMainWindow):
                 self.ui.lstRelations.addItem(i)
 
     def saveRelation(self):
-        if self.selectedRelation is None:
+        if not self.ui.lstRelations.selectedItems():
             r = QtWidgets.QApplication.translate(
                 "Form", "Select a relation first."
             )
@@ -268,7 +287,9 @@ class relForm(QtWidgets.QMainWindow):
         )[0]
         if (len(filename) == 0):  # Returns if no file was selected
             return
-        self.selectedRelation.save(filename)
+
+        relname = self.ui.lstRelations.selectedItems()[0].text()
+        self.user_interface.relations[relname].save(filename)
 
     def unloadRelation(self):
         for i in self.ui.lstRelations.selectedItems():
